@@ -1,30 +1,3 @@
-## TODO: roxygen documentation of all functions
-
-# devtools::install_github("ropensci/plotly", ref = "fix/subplot")
-# reference: https://plot.ly/ggplot2/ggdendro-dendrograms/
-# to answer later: http://stackoverflow.com/questions/34733829/questions-about-a-tutorial-regarding-interactive-heatmaps-with-plotly
-# to check: https://plot.ly/r/heatmaps/
-
-
-
-#' @title Checks if an object is of class plotly or not.
-#' @export
-#' @description
-#' Helpful for the plot_method in link{heatmaply}.
-#'
-#' @param x an object to check
-#'
-#' @return
-#' TRUE if the object inherits "plotly" as a class.
-#'
-is.plotly <- function(x) {
-  inherits(x, "plotly")
-}
-
-
-
-
-
 #' @title  Cluster heatmap based on plotly
 #' @name heatmaply
 #'
@@ -97,17 +70,23 @@ is.plotly <- function(x) {
 #' Can accept alternative character strings indicating the
 #' method to be passed to hclustfun By default hclustfun is \link{hclust} hence
 #' this can be one of "ward.D", "ward.D2", "single", "complete", "average" (= UPGMA), "mcquitty" (= WPGMA), "median" (= WPGMC) or "centroid" (= UPGMC).
+#' Specifying hclust_method=NA causes heatmaply to use 
+#' \code{\link[dendextend]{find_dend}} to find the "optimal" dendrogram for
+#' the data.
 #'
 #' @param distfun_row distfun for row dendrogram only.
 #' @param hclustfun_row hclustfun for col dendrogram only.
 #' @param distfun_col distfun for row dendrogram only.
 #' @param hclustfun_col hclustfun for col dendrogram only.
 #'
-#' @param dendrogram character string indicating whether to draw 'none', 'row',
-#' 'column' or 'both' dendrograms. Defaults to 'both'. However, if Rowv (or Colv)
-#' is FALSE or NULL and dendrogram is 'both', then a warning is issued and Rowv
-#' (or Colv) arguments are honoured.
+#' @param dendrogram character string indicating whether to compute 'none', 
+#' 'row', 'column' or 'both' dendrograms. Defaults to 'both'. 
+#' However, if Rowv (or Colv) is FALSE or NULL and dendrogram is 'both', 
+#' then a warning is issued and Rowv (or Colv) arguments are honoured.
 #' It also accepts TRUE/FALSE as synonyms for "both"/"none".
+#' @param show_dendrogram Logical vector of length two, controlling whether 
+#' the row and/or column dendrograms are displayed. If a logical scalar is 
+#' provided, it is repeated to become a logical vector of length two.
 #' @param reorderfun function(d, w) of dendrogram and weights for reordering the
 #' row and column dendrograms. The default uses stats{reorder.dendrogram}
 #'
@@ -210,10 +189,12 @@ is.plotly <- function(x) {
 #' "GW" (Gruvaeus and Wainer heuristic to optimize the Hamiltonian path length
 #' that is restricted by the dendrogram structure)
 #'
-#' @param heatmap_layers ggplot object (eg, theme_bw()) to be added to
+#' @param heatmap_layers ggplot object(s) (eg, list(theme_bw())) to be added to
 #'  the heatmap before conversion to a plotly object.
-#' @param side_color_layers Layers to be added to side color plots, similar to
-#' 	heatmap_layers.
+#' @param side_color_layers ggplot2 objects to be added to side color plots,
+#'  similar to heatmap_layers.
+#' @param dendrogram_layers ggplot2 objects to be added to dendrograms,
+#'  similar to heatmap_layers and side_color_layers.
 #' @param branches_lwd numeric (default is 0.6). The width of the dendrograms' branches.
 #' If NULL then it is ignored. If the "lwd" is already defined in Rowv/Colv then this
 #' parameter is ignored (it is checked using \link[dendextend]{has_edgePar}("lwd")).
@@ -249,6 +230,9 @@ is.plotly <- function(x) {
 #' @param colorbar_len The length of the colorbar/color key relative to the total
 #' plot height. Only used if plot_method = "plotly"
 #'
+#' @param colorbar_thickness The thickness (width) of the colorbar/color key
+#' in pixels. Only used if plot_method = "plotly".
+#'
 #' @param colorbar_xanchor,colorbar_yanchor The x and y anchoring points of the
 #' colorbar/color legend. Can be "left", "middle" or "right" for colorbar_xanchor,
 #' and "top", "middle" or "bottom" for colorbar_yanchor.
@@ -291,6 +275,10 @@ is.plotly <- function(x) {
 #' backward compatibility with gplots::heatmap.2.
 #' @param dend_hoverinfo Boolean value which controls whether mouseover text 
 #' is shown for the row and column dendrograms.
+#' @param side_color_colorbar_len As with colorbar_len, this controls the
+#' length of the colorbar/color key relative to the total plot height. 
+#' This argument controls the colorbar_len of the side colour plots. 
+#' Only used if plot_method = "plotly".
 #'
 #' @export
 #' @examples
@@ -510,6 +498,7 @@ heatmaply.default <- function(x,
                               hclustfun_col,
 
                               dendrogram = c("both", "row", "column", "none"),
+                              show_dendrogram = c(TRUE, TRUE),
                               reorderfun = function(d, w) reorder(d, w),
 
                               k_row = 1,
@@ -544,6 +533,7 @@ heatmaply.default <- function(x,
                               seriate = c("OLO", "mean", "none", "GW"),
                               heatmap_layers = NULL,
                               side_color_layers = NULL,
+                              dendrogram_layers = NULL,
                               branches_lwd = 0.6,
                               file,
                               width = NULL,
@@ -557,6 +547,7 @@ heatmaply.default <- function(x,
                               subplot_widths = NULL,
                               subplot_heights = NULL,
                               colorbar_len = 0.3,
+                              colorbar_thickness = 30,
                               colorbar_xanchor = if (row_dend_left) "right" else "left",
                               colorbar_yanchor = "bottom",
                               colorbar_xpos = if (row_dend_left) -0.1 else 1.1,
@@ -571,10 +562,11 @@ heatmaply.default <- function(x,
                               labRow = NULL, labCol = NULL,
                               custom_hovertext = NULL,
                               col = NULL,
-                              dend_hoverinfo = TRUE) {
+                              dend_hoverinfo = TRUE,
+                              side_color_colorbar_len = 0.3) {
   if (!missing(long_data)) {
     if (!missing(x)) {
-      warning("x and long_data should not be used together")
+      stop("x and long_data should not be used together")
     }
     assert_that(
       ncol(long_data) == 3,
@@ -623,11 +615,16 @@ heatmaply.default <- function(x,
   }
   dendrogram <- match.arg(dendrogram)
 
-  if (!(is.data.frame(x) | is.matrix(x))) stop("x must be either a data.frame or a matrix.")
-
-  if (!missing(srtRow)) row_text_angle <- srtRow
-  if (!missing(srtCol)) column_text_angle <- srtCol
+  if (!(is.data.frame(x) | is.matrix(x))) {
+    stop("x must be either a data.frame or a matrix.")
+  }
 
+  if (!missing(srtRow)) {
+    row_text_angle <- srtRow
+  }
+  if (!missing(srtCol)) {
+    column_text_angle <- srtCol
+  }
   if (!is.null(ColSideColors)) {
     col_side_colors <- ColSideColors
   }
@@ -635,8 +632,12 @@ heatmaply.default <- function(x,
     row_side_colors <- RowSideColors
   }
 
-  if (!missing(cexRow)) fontsize_row <- if (is.numeric(cexRow)) cexRow * 10 else cexRow
-  if (!missing(cexCol)) fontsize_col <- if (is.numeric(cexCol)) cexCol * 10 else cexCol
+  if (!missing(cexRow)) {
+    fontsize_row <- if (is.numeric(cexRow)) cexRow * 10 else cexRow
+  }
+  if (!missing(cexCol)) {
+    fontsize_col <- if (is.numeric(cexCol)) cexCol * 10 else cexCol
+  }
 
   # TODO: maybe create heatmaply.data.frame heatmaply.matrix instead.
   #       But right now I am not sure this would be needed.
@@ -665,14 +666,14 @@ heatmaply.default <- function(x,
 
   if (!is.null(labRow)) {
     if (all(is.na(labRow))) {
-      showticklabels[2] <- FALSE
+      showticklabels[[2]] <- FALSE
     }
   } else {
     labRow <- rownames(x)
   }
   if (!is.null(labCol)) {
     if (all(is.na(labCol))) {
-      showticklabels[1] <- FALSE
+      showticklabels[[1]] <- FALSE
     }
   } else {
     labCol <- colnames(x)
@@ -711,6 +712,7 @@ heatmaply.default <- function(x,
     hclustfun_col = hclustfun_col,
 
     dendrogram = dendrogram,
+    show_dendrogram = show_dendrogram,
     reorderfun = reorderfun,
 
     k_row = k_row,
@@ -747,6 +749,7 @@ heatmaply.default <- function(x,
     col_side_palette = col_side_palette,
     heatmap_layers = heatmap_layers,
     side_color_layers = side_color_layers,
+    dendrogram_layers = dendrogram_layers,
     ColSideColors = ColSideColors,
     RowSideColors = RowSideColors,
     branches_lwd = branches_lwd,
@@ -761,6 +764,7 @@ heatmaply.default <- function(x,
     subplot_widths = subplot_widths,
     subplot_heights = subplot_heights,
     colorbar_len = colorbar_len,
+    colorbar_thickness = colorbar_thickness,
     colorbar_xanchor = colorbar_xanchor,
     colorbar_yanchor = colorbar_yanchor,
     colorbar_xpos = colorbar_xpos,
@@ -771,7 +775,8 @@ heatmaply.default <- function(x,
     node_type = node_type,
     point_size_name = point_size_name,
     label_format_fun = label_format_fun,
-    dend_hoverinfo = dend_hoverinfo
+    dend_hoverinfo = dend_hoverinfo,
+    side_color_colorbar_len = side_color_colorbar_len
   )
 
   # TODO: think more on what should be passed in "..."
@@ -828,6 +833,7 @@ heatmaply.heatmapr <- function(x,
                                RowSideColors,
                                heatmap_layers = NULL,
                                side_color_layers = NULL,
+                               dendrogram_layers = NULL,
                                branches_lwd = 0.6,
                                label_names = c("row", "column", "value"),
                                fontsize_row = 10,
@@ -839,6 +845,7 @@ heatmaply.heatmapr <- function(x,
                                colorbar_xpos = if (row_dend_left) -0.1 else 1.1,
                                colorbar_ypos = 0,
                                colorbar_len = 0.3,
+                               colorbar_thickness = 30,
                                showticklabels = c(TRUE, TRUE),
                                dynamicTicks = FALSE,
                                node_type = c("scatter", "heatmap"),
@@ -847,7 +854,9 @@ heatmaply.heatmapr <- function(x,
                                point_size_name = "Point size",
                                label_format_fun = function(...) format(..., digits = 4),
                                custom_hovertext = x[["matrix"]][["custom_hovertext"]],
-                               dend_hoverinfo = TRUE) {
+                               dend_hoverinfo = TRUE,
+                               side_color_colorbar_len = 0.3) {
+
   node_type <- match.arg(node_type)
   plot_method <- match.arg(plot_method)
   cellnote_textposition <- match.arg(
@@ -933,10 +942,11 @@ heatmaply.heatmapr <- function(x,
     if (plot_method == "ggplot") {
       col_ggdend <- as.ggdend(cols)
       xlims <- c(0.5, nrow(col_ggdend$labels) + 0.5)
-      py <- ggplot(cols, labels = FALSE) + 
+      py <- ggplot(cols, labels = FALSE, na.rm = TRUE) + 
         theme_bw() +
         coord_cartesian(expand = FALSE, xlim = xlims) +
-        theme_clear_grid_dends
+        theme_clear_grid_dends +
+        dendrogram_layers
     } else {
       py <- plotly_dend(cols, 
                         side = "col", 
@@ -950,11 +960,11 @@ heatmaply.heatmapr <- function(x,
       row_ggdend <- as.ggdend(rows)
       ylims <- c(0.5, nrow(row_ggdend$labels) + 0.5)
 
-      px <- ggplot(row_ggdend, labels = FALSE) +
+      px <- ggplot(row_ggdend, labels = FALSE, na.rm = TRUE) +
         theme_bw() +
         coord_flip(expand = FALSE, xlim = ylims) +
-        theme_clear_grid_dends
-
+        theme_clear_grid_dends +
+        dendrogram_layers
       if (row_dend_left) {
         px <- px + scale_y_reverse()
       }
@@ -981,7 +991,8 @@ heatmaply.heatmapr <- function(x,
       row_dend_left = row_dend_left,
       label_names = label_names,
       type = node_type,
-      fontsize_row = fontsize_row, fontsize_col = fontsize_col,
+      fontsize_row = fontsize_row, 
+      fontsize_col = fontsize_col,
       point_size_mat = point_size_mat,
       point_size_name = point_size_name,
       label_format_fun = label_format_fun,
@@ -999,6 +1010,7 @@ heatmaply.heatmapr <- function(x,
       colorbar_yanchor = colorbar_yanchor, colorbar_xanchor = colorbar_xanchor,
       colorbar_xpos = colorbar_xpos, colorbar_ypos = colorbar_ypos,
       colorbar_len = colorbar_len,
+      colorbar_thickness = colorbar_thickness,
       custom_hovertext = custom_hovertext
     )
   }
@@ -1039,6 +1051,7 @@ heatmaply.heatmapr <- function(x,
         palette = row_side_palette,
         fontsize = fontsize_col,
         is_colors = !is.null(RowSideColors),
+        colorbar_len = side_color_colorbar_len,
         label_name = label_names[[1]]
       )
     }
@@ -1047,8 +1060,6 @@ heatmaply.heatmapr <- function(x,
   if (is.null(col_side_colors)) {
     pc <- NULL
   } else {
-    warning("The hover text for col_side_colors is currently not implemented (due to an issue in plotly). We hope this would get resolved in future releases.")
-
     side_color_df <- x[["col_side_colors"]]
     if (is.matrix(side_color_df)) side_color_df <- as.data.frame(side_color_df)
     assert_that(
@@ -1062,7 +1073,7 @@ heatmaply.heatmapr <- function(x,
         text_angle = row_text_angle,
         palette = col_side_palette,
         is_colors = !is.null(ColSideColors),
-        fontsize = fontsize_col,
+        fontsize = fontsize_row,
         label_name = label_names[[2]]
       ) + side_color_layers
     } else {
@@ -1071,8 +1082,9 @@ heatmaply.heatmapr <- function(x,
         type = "column",
         text_angle = row_text_angle,
         palette = col_side_palette,
-        fontsize = fontsize_col,
+        fontsize = fontsize_row,
         is_colors = !is.null(ColSideColors),
+        colorbar_len = side_color_colorbar_len,
         label_name = label_names[[2]]
       )
     }
@@ -1095,7 +1107,14 @@ heatmaply.heatmapr <- function(x,
   # turn p, px, and py to plotly objects if necessary
   if (!is.plotly(p)) {
     p <- ggplotly(p, dynamicTicks = dynamicTicks, tooltip = "text") %>%
-      layout(showlegend = TRUE)
+      layout(showlegend = FALSE)
+      ## Currently broken, see:
+      ##  https://github.com/ropensci/plotly/issues/1701
+      # %>%
+      # colorbar(
+      #   len = colorbar_len,
+      #   thickness = colorbar_thickness
+      # )
   }
 
   if (draw_cellnote) {
@@ -1104,18 +1123,18 @@ heatmaply.heatmapr <- function(x,
       cellnote_color <- predict_colors(p, plot_method = plot_method)
     }
 
-    df <- as.data.frame(x[["cellnote"]])
-    df$row <- 1:nrow(df)
-    mdf <- reshape2::melt(df, id.vars = "row")
+    df <- as.data.frame(x[["matrix"]][["cellnote"]])
+    df[["_row"]] <- seq_len(nrow(df))
+    mdf <- reshape2::melt(df, id.vars = "_row")
     ## TODO: Enforce same dimnames to ensure it's not scrambled?
     # mdf$variable <- factor(mdf$variable, levels = p$x$layout$xaxis$ticktext)
-    mdf$variable <- as.numeric(as.factor(mdf$variable))
+    mdf$variable <- as.factor(mdf$variable)
+    mdf$variable <- as.numeric(mdf$variable)
     mdf$value <- factor(mdf$value)
-
     p <- p %>% add_trace(
       data = mdf,
       x = ~ variable,
-      y = ~ row,
+      y = ~ `_row`,
       text = ~ value,
       inherit = FALSE,
       type = "scatter",
@@ -1154,8 +1173,6 @@ heatmaply.heatmapr <- function(x,
   )
   if (hide_colorbar) {
     p <- hide_colorbar(p)
-    # px <- hide_colorbar(px)
-    # py <- hide_colorbar(py)
   }
 
   # Adjust top based on whether main is empty or not.
@@ -1176,9 +1193,15 @@ heatmaply.heatmapr <- function(x,
   # add a white grid
   if (grid_gap > 0) {
     p <- style(p, xgap = grid_gap, ygap = grid_gap, traces = 1)
-    # doesn't seem to work.
-    # if(!is.null(pr)) pr <- style(pr, xgap = grid_gap)
-    # if(!is.null(pc)) pc <- style(pc, ygap = grid_gap)
+    # doesn't seem to work for ggplot2.
+    if (plot_method == "plotly") {
+      if (!is.null(pr)) {
+        pr <- style(pr, ygap = grid_gap)
+      }
+      if (!is.null(pc)) {
+        pc <- style(pc, xgap = grid_gap)
+      }
+    }
   }
 
 
@@ -1205,10 +1228,19 @@ heatmaply.heatmapr <- function(x,
   }
 
   heatmap_subplot <- heatmap_subplot_from_ggplotly(
-    p = p, px = px, py = py,
-    row_dend_left = row_dend_left, subplot_margin = subplot_margin,
-    widths = subplot_widths, heights = subplot_heights,
-    titleX = titleX, titleY = titleY, pr = pr, pc = pc, plot_method = plot_method
+    p = p, 
+    px = px, 
+    py = py,
+    row_dend_left = row_dend_left, 
+    subplot_margin = subplot_margin,
+    widths = subplot_widths, 
+    heights = subplot_heights,
+    titleX = titleX, 
+    titleY = titleY, 
+    pr = pr, 
+    pc = pc, 
+    plot_method = plot_method,
+    showticklabels = showticklabels
   )
   l <- layout(
     heatmap_subplot,
@@ -1230,46 +1262,36 @@ heatmaply.heatmapr <- function(x,
 
 
 
-
+default_dims <- function(px, pr) {
+  if (!is.null(px)) {
+    if (is.null(pr)) {
+      widths <- c(0.8, 0.2)
+    } else {
+      widths <- c(0.7, 0.1, 0.2)
+    }
+  } else {
+    if (is.null(pr)) {
+      widths <- 1
+    } else {
+      widths <- c(0.9, 0.1)
+    }
+  }
+  widths
+}
 
 
 heatmap_subplot_from_ggplotly <- function(p, px, py, pr, pc,
                                           row_dend_left = FALSE, subplot_margin = 0,
                                           titleX = TRUE, titleY = TRUE,
                                           widths=NULL, heights=NULL,
-                                          plot_method) {
-  if (is.null(widths)) {
-    if (!is.null(px)) {
-      if (is.null(pr)) {
-        widths <- c(0.8, 0.2)
-      } else {
-        widths <- c(0.7, 0.1, 0.2)
-      }
-    } else {
-      if (is.null(pr)) {
-        widths <- 1
-      } else {
-        widths <- c(0.9, 0.1)
-      }
-    }
-    if (row_dend_left) widths <- rev(widths)
+                                          plot_method,
+                                          showticklabels=c(TRUE, TRUE)) {
+  widths <- widths %||% default_dims(px, pr)
+  if (row_dend_left) {
+    widths <- rev(widths)
   }
+  heights <- heights %||% rev(default_dims(py, pc))
 
-  if (is.null(heights)) {
-    if (!is.null(py)) {
-      if (is.null(pc)) {
-        heights <- c(0.2, 0.8)
-      } else {
-        heights <- c(0.2, 0.1, 0.7)
-      }
-    } else {
-      if (is.null(pc)) {
-        heights <- 1
-      } else {
-        heights <- c(0.1, 0.9)
-      }
-    }
-  }
 
   # make different plots based on which dendrogram and sidecolors we have
   row1_list <- list(py, plotly_empty(), plotly_empty())
@@ -1319,8 +1341,10 @@ heatmap_subplot_from_ggplotly <- function(p, px, py, pr, pc,
         plots,
         nrows = nrows,
         widths = widths,
-        shareX = TRUE, shareY = TRUE,
-        titleX = titleX, titleY = titleY,
+        shareX = TRUE, 
+        shareY = TRUE,
+        titleX = titleX, 
+        titleY = titleY,
         margin = subplot_margin,
         heights = heights
       )
@@ -1334,7 +1358,7 @@ heatmap_subplot_from_ggplotly <- function(p, px, py, pr, pc,
       l <- list(
         anchor = paste0("x", str),
         side = "right",
-        showticklabels = TRUE
+        showticklabels = showticklabels[[2]]
       )
       num_cols <- sum(!ind_null_col)
       if (num_cols == 1) {
@@ -1367,3 +1391,17 @@ calc_margin <- function(labels, fontsize) {
   max(nchar(labels) * fontsize, na.rm = TRUE) * 0.6
   # http://stackoverflow.com/questions/19113725/what-dependency-between-font-size-and-width-of-char-in-monospace-font
 }
+
+#' @title Checks if an object is of class plotly or not.
+#' @export
+#' @description
+#' Helpful for the plot_method in link{heatmaply}.
+#'
+#' @param x an object to check
+#'
+#' @return
+#' TRUE if the object inherits "plotly" as a class.
+#'
+is.plotly <- function(x) {
+  inherits(x, "plotly")
+}
